
# q-expansion of newform 5776.2.a.m, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5776
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [0], [1], [6], [-5], [3], [0], [-3], [-9], [-4], [-2], [0], [-8], [0], [3], [9], [-10], [5], [-6], [-7], [-10], [6], [12], [10], [18], [14], [-9], [-11], [-6], [2], [0], [-9], [4], [0], [-10], [-22], [-20], [12], [-6], [0], [-2], [-3], [-14], [0], [-11], [5], [26], [-15], [-22], [-6], [21], [-8], [-6], [-12], [-24], [6], [-11], [8], [0], [22], [21], [20], [21], [-19], [9], [-1], [4], [-18], [-10], [-15], [-21], [28], [-23], [-7], [18], [18], [20], [0], [-32], [12], [-17], [6], [-2], [-28], [18], [18], [17], [-12], [4], [-18], [-36], [2], [36], [4], [21], [-30], [36], [11], [2], [44], [24], [-12], [24], [4], [11], [12], [-30], [-24], [28], [-22], [2], [-6], [10], [16], [-6], [22], [-27], [24], [-45], [13], [-44], [33], [36], [10], [12], [-10], [-39], [37], [32], [16], [-36], [-40], [2], [-21], [5], [-51], [-31], [39], [9], [11], [0], [-41], [33], [-11], [-48], [-46], [12], [-14], [-18], [-23], [-18], [34], [-42], [-37], [48], [7], [33], [-7], [-21], [48], [-30], [-32], [12], [12], [-30], [20], [8], [-20], [-24], [39], [-5], [0], [-1], [32], [30], [-8], [30], [56], [8], [-53], [-33], [22], [-48], [-51], [-6], [61], [8], [58], [-27], [34], [36], [56], [-51], [12], [-12], [41], [44], [3], [27], [-30], [8], [-5], [28], [6], [-48], [-47], [-15], [-3], [-67], [29], [-24], [28], [27], [60], [46], [-23], [-30], [18], [18], [58], [14], [-18], [61], [27], [62], [-66], [-42], [-10], [-66], [14], [29], [-40], [30], [-26], [39], [-58], [72], [-18], [-12], [-12], [-74], [31], [43], [-36], [60], [16], [-60], [-4], [57], [-58], [-69], [57], [-62], [-12], [30], [2], [-28], [42], [38], [-28], [60], [32], [7], [66], [17], [45], [3], [-13], [66], [-5], [47], [-55], [-5], [-14], [-29], [-6], [-53], [-20], [0], [-66], [-23], [-27], [-17], [4], [-24], [-79], [6], [19], [18], [66], [-36], [-66], [60], [46], [-12], [26], [6], [3], [46], [14], [-39], [25], [69], [4], [34], [81], [-23], [24], [20], [-48], [30], [-90], [-4], [24], [16], [54], [-18], [-37], [78], [-37], [23], [39], [-70], [-9], [-2], [-7], [-73], [24], [-12], [80], [9], [1], [-30], [-74], [48], [-47], [-24], [-49], [43], [-71], [-57], [15], [-10], [27], [84], [2], [-37], [-66], [-72], [-40], [-2], [48], [8], [22], [-72], [9], [-48], [-33], [-54], [8], [-90], [42], [-15], [-74], [-44], [-42], [74], [-38], [-90], [-61], [9], [39], [4], [-16], [-45], [-72], [38], [-3], [-44], [78], [-30], [-65], [-66], [5], [-42], [-17], [-4], [34], [24], [82], [-87], [54], [10], [36], [-14], [74], [-48], [68], [-72], [-14], [66], [-94], [72], [-6], [-11], [-82], [-84], [23], [24], [64], [54], [-27], [-62], [77], [36], [6], [-28], [51], [48], [-42], [-83], [15], [-69], [-2], [-87], [21], [3], [88], [21]]
